/***** drawTree.c *********************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Wed Jun  7 15:17:17 2017
 **************************************************/
#include <ctype.h>
#include "drawTree.h"

int globalX, globalDepth;

void printOutputSet(FILE *fp, KNode *np){
  KNode *tnp;
  int angle;

  if(np->child != NULL)
    printOutputSet(fp, np->child);

  if(np->id > -1){
    if(np->child == NULL)
      angle = -90;
    else
      angle = 0;
    fprintf(fp, "%%%% \\nput{%d}{%d}{\\{%d", angle, np->nId, np->id+1);
    tnp = np->fail;
    while(tnp->depth > 0){
      if(tnp->id > -1)
	fprintf(fp, ",%d", tnp->id+1);
      tnp = tnp->fail;
    }
    fprintf(fp, "\\}}\n");
  }

  if(np->sibling != NULL)
    printOutputSet(fp, np->sibling);
}

void printNodeLabels(FILE *fp, KNode *np){
  
  if(np->child != NULL)
    printNodeLabels(fp, np->child);

  if(np->id > -1){
    if(np->child == NULL)
      fprintf(fp, "\\nput{-90}{%d}{%d}\n", np->nId, np->id+1);
    else
      fprintf(fp, "\\nput{0}{%d}{%d}\n", np->nId, np->id+1);
  }

  if(np->sibling != NULL)
    printNodeLabels(fp, np->sibling);
}

void printFailureLinks(FILE *fp, KNode *np){
  int angle;

  if(np->child != NULL)
    printFailureLinks(fp, np->child);

  if(np->fail != NULL){
    if(np->depth > 0){
      if(np->x > np->fail->x)
	angle = -50;
      else
	angle = 50;
      fprintf(fp, "\\ncarc[arcangle=%d]{->}{%d}{%d}\n", angle, np->nId, np->fail->nId);
    }
  }
  if(np->sibling != NULL)
    printFailureLinks(fp, np->sibling);
    
}

void printEdges(FILE *fp, KNode *np){
  KNode *cp;
  float x1, x2;
  int y1, y2;

  if(np->child != NULL)
    printEdges(fp, np->child);
  if(np->child){
    x1 = np->x;
    y1 = globalDepth - np->depth;
    y2 = globalDepth - np->child->depth;
    cp = np->child;
    fprintf(fp, "\\cnode(%g, %d){1pt}{%d}\n",x1, y1, np->nId);
    while(cp != NULL){
      x2 = cp->x;
      fprintf(fp, "\\pstextpath[c]{\\psline{->}(%g,%d)(%g,%d)}{\\texttt{%c}}\n", x1, y1, x2, y2, cp->c);
      cp = cp->sibling;
    }
  }

  if(np->sibling != NULL)
    printEdges(fp, np->sibling);
}

void printNodes(FILE *fp, KNode *np){
  if(np->child != NULL)
    printNodes(fp,np->child);
  
  fprintf(fp,"\\dotnode(%g,%d){%d}\n",np->x,globalDepth-np->depth,np->nId);

  if(np->sibling != NULL)
    printNodes(fp,np->sibling);
}

void setLeafX(KNode *np){
  if(np->child != NULL)
    setLeafX(np->child);
  if(np->sibling != NULL)
    setLeafX(np->sibling);
  if(np->child == NULL)
    np->x = globalX++;
}

void setInternalX(KNode *np){
  float min, max, n;
  KNode *cp;

  if(np->child != NULL)
    setInternalX(np->child);

  if(np->child != NULL){
    cp = np->child;
    min = cp->x;
    n = 1;
    while(cp->sibling != NULL){
      cp = cp->sibling;
      n++;
    }
    max = cp->x;
    np->x = min + (max - min) / 2.;
  }
  if(np->depth > globalDepth)
    globalDepth = np->depth;

  if(np->sibling != NULL)
    setInternalX(np->sibling);
}

void drawTree(FILE *fp, KNode *root){
  globalX = 0;
  globalDepth = -1;
  setLeafX(root);
  setInternalX(root);
  fprintf(fp, "\\begin{pspicture}(0,-0.5)(%d,%g)\n",globalX,globalDepth+0.2);
  fprintf(fp, "\\psset{linecolor=lightgray}\n");
  printNodes(fp, root);
  printEdges(fp, root);
  fprintf(fp, "\\psset{linewidth=0.5pt,linecolor=orange,nodesepA=2pt,nodesepB=2pt}\n");
  fprintf(fp, "%%%%%%%% Failure links %%%%%%%%\n");
  printFailureLinks(fp, root);
  fprintf(fp, "%%%%%%%% Node labels %%%%%%%%\n");
  printNodeLabels(fp, root);
  fprintf(fp, "%%%%%%%% Output sets %%%%%%%%\n");
  printOutputSet(fp, root);
  fprintf(fp, "\\end{pspicture}\n");
}

void printLatexHeader(FILE *fp){
  fprintf(fp,"\\documentclass{article}\n");
  fprintf(fp,"\\usepackage{pst-all,graphics,times}\n");
  fprintf(fp,"\\oddsidemargin=0cm\n");
  fprintf(fp,"\\evensidemargin=0cm\n");
  fprintf(fp,"\\pagestyle{empty}\n");
  fprintf(fp,"\\textwidth=16cm\n");
  fprintf(fp,"\\textheight=23cm\n\n");
  fprintf(fp,"\\begin{document}\n");
  fprintf(fp,"\\title{Keyword Tree with Failure Links}\\author{}\\date{}\\maketitle\n");
}
