/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "interface.h"
#include "sequenceData.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  int c;
  char *optString = "hrisvt:p:f:";

  args = (Args *)emalloc(sizeof(Args));
  args->p = NULL;
  args->f = NULL;
  args->r = 0;
  args->i = 0;
  args->s = 0;
  args->h = 0;
  args->v = 0;
  args->e = 0;
  args->t = NULL;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 'p':                           /* patterns */
      args->p = estrdup(optarg);
      break;
    case 'f':                           /* name of pattern file */
      args->f = estrdup(optarg);
      break;
    case 'r':                           /* include reverse strand? */
      args->r = 1;
      break;
    case 'i':                           /* ignore case? */
      args->i = 1;
      break;
    case 't':                           /* tree file */
      args->t = estrdup(optarg);
      break;
    case 's':                           /* load sequences separately */
      args->s = 1;
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  if(!args->p && !args->f && !args->v && !args->h){
    fprintf(stderr, "ERROR: Please specify one or more patters using either the -p or the -f option.\n");
    args->e = 1;
  }
    
  return args;
}


void printUsage(char *version){
  printf("Usage: %s [-p <PATTERN> | -f <patterns.fasta>] [options] [inputFiles]\n",progname());
  printf("Match a set of motifs against one or more sequences\n");
  printf("Example: ./keywordMatcher -p AACGTTCA mgGenome.fasta\n");
  printf("Options (need to specify either -p or -f):\n");
  printf("\t[-p <STRING> delineated by | and enclosed by quotes, e.g. 'ACCGT|AGACCG']\n");
  printf("\t[-f <FILE> file of patterns in FASTA format]\n");
  printf("\t[-t <FILE> file for printing keyword tree]\n");
  printf("\t[-r include reverse strand in search; default: only forward]\n");
  printf("\t[-i ignore case; default: case-sensitive]\n");
  printf("\t[-s load sequences separately - saves memory but slower; default: load all sequences at once]\n");
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  printf("Limits:\n");
  printf("\tmaximum length of line in pattern file: %d\n", SEQLINE);
  printf("\tmaximum number of patterns that can be handled: %d\n", MAX_PATTERNS);
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
