/***** keywordMatcher.c *******************************************
 * Description: Match a set of patterns against a text using a
 *    keyword tree. 
 * Reference: Gusfield, D. (1997). Algorithms on Strings, Trees,
 *    and sequences; Computer Science and Computational Biology.
 *    CUP, p. 52ff.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Fri Aug 27 19:30:32 2004. 
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <assert.h>
#include "eprintf.h"
#include "stringUtil.h"
#include "keywordMatcher.h"
#include "ktree.h"
#include "construct.h"
#include "drawTree.h"

char *getWrapperFileName(char *fileName){
  char *buffer;
  int i;

  buffer = (char *)emalloc(1024 * sizeof(char));
  i = 0;
  while(fileName[i] != '.' && fileName[i] != '\0'){
    buffer[i] = fileName[i];
    i++;
  }
  buffer[i] = '\0';
  strcat(buffer, "Wrapper.tex");
  return buffer;
}

char *getDvipsFileName(char *fileName){
  char *buffer;
  int i;

  buffer = (char *)emalloc(1024 * sizeof(char));
  i = 0;
  while(fileName[i] != '.' && fileName[i] != '\0'){
    buffer[i] = fileName[i];
    i++;
  }
  buffer[i] = '\0';
  strcat(buffer, ".dvi");
  return buffer;
}

char *getPsFileName(char *fileName){
  char *buffer;
  int i;

  buffer = (char *)emalloc(1024 * sizeof(char));
  i = 0;
  while(fileName[i] != '.' && fileName[i] != '\0'){
    buffer[i] = fileName[i];
    i++;
  }
  buffer[i] = '\0';
  strcat(buffer, ".ps");
  return buffer;
}

int main(int argc, char *argv[]){
  int i;
  Args *args;          /* arguments */
  char *version;       /* program version */
  FILE *fp;
  KTree *ktree;
  char *wrapperFile, *dvipsFile, *psFile;

  version = "1.20";
  setprogname2("keywordMatcher");
  args = getArgs(argc, argv);
  if(args->v)
    printSplash(version);
  if(args->h || args->e)
    printUsage(version);
  ktree = getKeywordTree(args);
  if(args->t){
    wrapperFile = getWrapperFileName(args->t);
    dvipsFile = getDvipsFileName(wrapperFile);
    psFile = getPsFileName(wrapperFile);
    fp = efopen(wrapperFile, "w");
    printLatexHeader(fp);
    fprintf(fp,"\\begin{center}\n\\input{%s}\n\\end{center}\n\\end{document}",args->t);
    fclose(fp);
    fp = efopen(args->t,"w");
    drawTree(fp, ktree->root);
    fclose(fp);
    fprintf(stderr, "# The keyword tree is contained in %s; to view it, either include it in your document,\n",args->t);
    fprintf(stderr, "#    or run latex %s; dvips %s to generate %s\n",wrapperFile,dvipsFile,psFile);
    fprintf(stderr, "# To print output-set instead of node labels: adjust commenting at end of %s\n",args->t);
  }
  if(args->numInputFiles == 0){
    fp = stdin;
    searchSingleSeq(fp, args, ktree);
  }else{
    for(i=0; i<args->numInputFiles; i++){
      fp = efopen(args->inputFiles[i], "r");
      searchMultipleSeq(fp, args, ktree);
      fclose(fp);
    }
  }
  free(args);
  free(progname());
  return 0;
}
