/***** preprocess.c ***********************************************
 * Description: Preprocess keyword tree.
 * Reference: Gusfield, D. (1997). Algorithms on Strings, Trees,
 *    and sequences; Computer Science and Computational Biology.
 *    CUP, p. 58.
 * TO DO: Implement the comment made in exercise 13. on p. 67
 *    of Gusfield (1997).
 * Author: Bernhard Haubold, haubold@evolbio.mgp.de
 * File created on Wed Sep  8 10:20:36 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "keywordMatcher.h"

/*
 * Add failure and output links to tree during a breadth-first
 * traversal.
 * Reference: Gusfield, p. 60. 
 */
void preprocessTree(KNode *root){
  KNode *w, *wdash, *v, *vdash;
  KNode *child, *front, *back;
  char x;

  front = back = root;
  front->fail = NULL;
  front->out = NULL;
  while(front != NULL){
    v = front;
    vdash = v->out;
    /* add children of current node to queue */
    for(child=v->child; child != NULL; child=child->sibling){
      child->out = v;
      back->fail = child;
      back = child;
    }
    /* nullify end of queue */
    back->fail = NULL;
    /* move along one step in queue */
    front = front->fail;
    v->fail = v->out = NULL;
    if(v != NULL && vdash != NULL){
      /* set failure link */
      if(v == root){
	v->fail = v;
      }else if(vdash == root){
	v->fail = vdash;
      }else{
	wdash = NULL;
	x = v->c;
	w = vdash->fail;
	while((wdash = findChild(w,x)) == NULL && w != root)
	  w = w->fail;
	if(wdash != NULL && wdash->c == x )
	  v->fail = wdash;
	else
	  v->fail = root;
      }
    }
    /* set output link */
    if(v->fail == NULL)
      ;
    else if(v->fail->id > -1)
      v->out = v->fail;
    else if(v->fail->out != NULL)
      v->out = v->fail->out;
  }
  root->fail = root;
}
